unit UnAstCoT; { types and constants }

interface

Const
  jd2000   = 2451545.0 ;
  julianc  =   36525.0 ;
  difepoch = 2400000.5 ;

Const
  GaussConst       = 0.0172020989500000e0 ;
                  { in AstrUnit**3/((Ephemer.Day**2)*MassOfSun if in Sqr }
  CavendishConst   = 6.672e-20 ;     { in km**3/(kg*s**2) }
  AstrUnit         = 1.49597870691e8 ;  { in km }
  VelOfLight       = 299792.4580 ;   { in km/s }
  SunFM            = 1.32712438e11 ; { in km**3/s**2 }
  GeoR0             = 6378.140 ; { equatorial radius of the Earth in km }

const
  grarad  = Pi/180 ;
  radgra  = 180/Pi ;
  secrad  = grarad/3600.0 ;
  radsec  = 3600.0*radgra ;
  pi22    = 2*Pi ;
  twopi   = pi22 ;

type
  tastel  = record { to identify the current minor planet }
              snm   : string[6] ;  { may be the number of the asteroid }
              stn   : string[11] ; { may be the name }
              epo   : double ;   { elements epoch in julian day }
              ela   : double ;   { semimajor axis in astronomical unit }
              ele   : double ;   { eccentricity }
              eli   : double ;   { inclination in degree }
              eln   : double ;   { longitude of ascension node in degree }
              elp   : double ;   { argument of perihelii in degree }
              elm   : double ;   { mean anomaly in degree }
              prh   : single ;   { parametr H }
              prg   : single ;   { parametr G }
            end;

type
  tvect3    = array[1..3] of extended ; { for position and velosity }
  TypeDim3  = tvect3 ;                  { the same }
  tmatr33   = array[1..3,1..3] of extended ; { matrix to change }
  typedim33 = tmatr33 ;                      { the same }
  TFundArg  = array [1..5] of Extended ; { fundamental arguments }

type
  TCooRec   = record
                nums  : Integer ;    { number of station in list }
                name  : string[12] ; { name   of station in list }
                desc  : tvect3 ;     { descart positions in kilometer }
                vauc  : tvect3 ;     { positions in Astronomical unit }
                geod  : typedim3 ;   { lat long degree h kilometer }
                topm  : tmatr33 ;    { matrix to topocentric }
              end;

type
  terand    = record  { to model with random error in arc second }
                eras  : extended ;  { random error for right ascension }
                erds  : extended ;  { random error for declination }
                alti  : extended ;  { altitude in degree }
              end;

implementation

end.
