
{ to model observations
  by using numerical integration method }

(*
type
  tastel  = record { to identify the current minor planet }
              snm   : string[5] ;  { may be the number of the asteroid }
              stn   : string[11] ; { may be the name }
              epo   : double ;   { elements epoch in julian day }
              ela   : double ;   { semimajor axis in astronomical unit }
              ele   : double ;   { eccentricity }
              eli   : double ;   { inclination in degree }
              eln   : double ;   { longitude of ascension node in degree }
              elp   : double ;   { argument of perihelion in degree }
              elm   : double ;   { mean anomaly in degree }
              prh   : single ;   { parametr H }
              prg   : single ;   { parametr G }
            end;
*)

unit UnAstMob ;

interface

uses
  UnAstCoT ; { for type tastel record for elements }

{ to get ephemeris by integration orbit }

procedure SingleAstEphem ( ia  : Byte ;      { index what to do }
                           ta  : Extended ;  { start  moment }
                           tb  : Extended ;  { finish moment }
                           ds  : Extended ;  { step in day for result }
                           el  : tastel ;    { UnAstCoT type elements }
                           pc  : tcoorec ) ; { unastcot type station }

implementation

uses
  uneveras , { for EverGrator procedures EverInit EverCoefNullo }
  unforequ , { for procedure  toequars ecliptic elem to equapos }
  unastrng , { for function AstFromEarth }
  unastres ; { for procedure  SingleAstResult }

function ToGetCurStep ( dr  : extended ) : extended ;
begin
  ToGetCurStep:=1.0; { step in day default }
  if  dr < 0.005  { near the Earth }
    then
      ToGetCurStep:=0.001  { step in day }
    else
      if  dr < 0.05
        then
          ToGetCurStep:=0.01
        else
          if  dr < 0.1
            then
              ToGetCurStep:=0.1;
end;

{ to get ephemeris by integration orbit }

procedure SingleAstEphem ( ia  : Byte ;      { index what to do }
                           ta  : Extended ;  { start  moment }
                           tb  : Extended ;  { finish moment }
                           ds  : Extended ;  { step in day for result }
                           el  : tastel ;    { UnAstCoT type elements }
                           pc  : tcoorec ) ; { unastcot type station }
var
  tc  : extended ;       { current julian date }
  dt  : extended ;       { step in day to integrate }
  xc  : tvect3 ;         { type unastcot position }
  vc  : tvect3 ;         { type unastcot velosity }
  vm  : extended ;       { magnitude }
  dr  : extended ;       { range from the Earth in km }
  za  : extended ;       { small interval for integration start }
  zb  : extended ;       { small interval for integration finish }
  ra  : tvect3 ;         { small position for integration start }
  rb  : tvect3 ;         { small position for integration finish }
  va  : tvect3 ;         { small velosity for integration start }
  vb  : tvect3 ;         { small velosity for integration finish }
  xe  : tvect3 ;         { type unastcot position }

begin

  moprint:=0; { global unastres simple count to print }

  dt:=+1.0;   { this is a value of step in day to integrate }
              { in tc is epoch of the elements as start moment }
  tc:=el.epo; { this value may be greater than start moment ta }
                      { from elements ecliptic to equator }
  toequars(el,xc,vc); { unforequ position AU velosity AU/day }

  za:=tc; { start  moment to integrate }
  zb:=za; { finish moment to integrate initial equal start }

  ra:=xc; { start  position vector in interval to integrate }
  rb:=ra; { finish position vector in interval to integrate }

  va:=vc; { start  velosity vector in interval to integrate }
  vb:=va; { finish velosity vector in interval to integrate }

  EverInit ;       { unit ueveras to start PosVelEverGrator }
  EverCoefNullo ;  { unit ueveras to start PosVelEverGrator }

  if  za > ta                       { epoch of the elements }
    then                 { may be greater than start moment }
      while  za > ta  do         { backward to start moment }
        PosVelEverGrator(-dt,za,ra,va,zb,rb,vb)  { uneveras }
    else                 { or may be less than start moment }
      while  za < ta  do          { forward to start moment }
        PosVelEverGrator(+dt,za,ra,va,zb,rb,vb); { uneveras }

  za:=zb; { moment in julian day  forward }
  ra:=rb; { start position vector to integrate }
  va:=vb; { start velosity vector to integrate }

  tc:=za; { current moment to print result }

  EverInit ;      { unit uneveras to start PosVelEverGrator }
  EverCoefNullo ; { unit uneveras to start PosVelEverGrator }

  while  tc < tb  do { from start to finish mom }
    begin            { step by step and analyse }

      tc:=tc+ds; { one step to print result }

      while  tc < za  do { current moment to interval }
        tc:=tc+ds;       { to the next moment }

      while  ( zb < (tc-1.0e-8) )  do
        begin
          PosVelEverGrator(dt,za,ra,va,zb,rb,vb);    { from unit uneveras }
          dr:=AstFromEarth(zb,rb); { unastrng simple range from the Earth }
          dt:=ToGetCurStep(dr);    { to control step in day }
        end;

      SingleAstResult(ia,el,pc,tc,za,zb,ra,va);  { from unit unastres }

    end;

end;

end.
