
unit UnAstRes ; { to print some result }

interface

uses
  unastcot ; { types tvect3 position velosity tastel elements }

var
  moprint  : integer ; { global simple count for simple print }

{ to obtain vector of the object from station
  in topocentric horizontal system with light delay }

procedure VectorWithDelay ( tc  : Extended ;       { current moment }
                            pc  : TCooRec ;        { record of station }
                            za  : Extended ;       { start mom in int. }
                            ra  : tvect3 ;         { pos for start mom }
                            va  : tvect3 ;         { vel for start mom }
                            var  tz  : Extended ;  { moment with delay }
                            var  xz  : tvect3 ;    { vector topo equat }
                            var  xp  : tvect3 ;    { vect pos topohori }
                            var  bo  : Boolean ) ; { moment may be out }

{ to obtain current set
  of ecliptical heliocentric elements
  input 
  tz    current moment  in interval [ za , zb ]
  ra va current polinom in interval [ za , zb ] }

procedure EcliptelMom ( tz  : Extended ;      { current moment }
                        za , zb  : extended ; { interval }
                        ra , va  : tvect3 ;   { condition }
                        var  ec  : tastel ) ; { elements result }

procedure ToTryEphem ( el       : tastel ;   { for set of element }
                       pc       : tcoorec ;  { record for station }
                       tc       : extended ; { current moment }
                       za , zb  : extended ; { moments for interval }
                       ra , va  : tvect3 ) ; { start condition vect }

procedure ToTryElems ( el  : tastel ;        { for set of element }
                       tc  : extended ;      { current moment }
                       za , zb  : extended ; { moments for interval }
                       ra , va  : tvect3 ) ; { start condition vect }

{ to obtain some values
  during small interval of integration [ za , zb ] }

procedure SingleAstResult ( ia  : Byte ;          { index what to do }
                            el  : tastel ;        { for set of element }
                            pc  : tcoorec ;       { record for station }
                            tc  : extended ;      { current moment }
                            za , zb  : extended ; { moments for interval }
                            ra , va  : tvect3 ) ; { start condition vect }

implementation

uses
  uneveras , { procedure ClcVectFromPolinom }
  unforequ , { procedure toeclipt in AU/day }
  unastrng , { procedure AstFromEarth distance in AU }
  unastvam , { function  ToGetMau for the magnitude }
  unforcoo , { ClcTopCoorInDegree with refraction correction }
  unforday , { simple function CharDayOrNight }
  unforfun , { function  VectorModul }
  unfortim , { procedures for TDT TDB }
  unephjpl , { procedures for ephemer }
  unforprt , { protocol file fprt number point nprt }
  unstrdat ; { date from julan day to civil form in string }

{ to obtain vector of the object from station
  in topocentric horizontal system with light delay }

procedure VectorWithDelay ( tc  : Extended ;       { current moment }
                            pc  : TCooRec ;        { record of station }
                            za  : Extended ;       { start mom in int. }
                            ra  : tvect3 ;         { pos for start mom }
                            va  : tvect3 ;         { vel for start mom }
                            var  tz  : Extended ;  { moment with delay }
                            var  xz  : tvect3 ;    { vector topo equat }
                            var  xp  : tvect3 ;    { vect pos topohori }
                            var  bo  : Boolean ) ; { moment may be out }
var
  dc  : Extended ; { difference inner interval in day }
  xc  : tvect3 ;   { position AU }
  vc  : tvect3 ;   { velosity AU/day }
begin              { vector for ephemeris xz xp refer to station }
  bo:=false;  { to prevent strange for approximation }
  tz:=tc;                    { xz in equatorial xp in horizontal }
  if  tz < za    { moment may be out of our interval }
    then  Exit ; { of approximation boolean is false }
  dc:=tz-za; { difference in day from start moment za }
  ClcVectFromPolinom(dc,ra,va,xc,vc); { uneveras }
  OstatVector(tc,xc,pc,xz,xp); { unastrng to station AU }
  { with light delay in day constants from UnAstCoT }
  tz:=tc-VectorModul(xp)*AstrUnit/(86400.0*VelOfLight);
  if  tz < za    { moment may be out of our interval }
    then  Exit ; { of approximation boolean is false }
  dc:=tz-za; { difference in day from start moment za }
  ClcVectFromPolinom(dc,ra,va,xc,vc); { uneveras }
  OstatVector(tc,xc,pc,xz,xp);  { unastrng to station AU }
  { with light delay in day constants from UnAstCoT }
  tz:=tc-VectorModul(xp)*AstrUnit/(86400.0*VelOfLight);
  if  tz < za    { moment may be out of our interval }
    then  Exit ; { of approximation boolean is false }
  dc:=tz-za; { difference in day from start moment za }
  ClcVectFromPolinom(dc,ra,va,xc,vc); { uneveras }
  OstatVector(tc,xc,pc,xz,xp);  { unastrng to station AU }
  bo:=true; { moment tz is good in approximation interval }
end;

{ to obtain current set
  of ecliptical helocentric elements
  input
  tz    current moment  in interval [ za , zb ]
  ra va current polinom in interval [ za , zb ] }

procedure EcliptelMom ( tz  : Extended ;      { current moment }
                        za , zb  : extended ; { interval }
                        ra , va  : tvect3 ;   { condition }
                        var  ec  : tastel ) ; { elements result }
var
  dc  : Extended ; { difference in day inner interval }
  xc  : tvect3 ;   { position }
  vc  : tvect3 ;   { velosity }
begin                         
  dc:=tz-za;    { difference in day from start moment za }
  if  dc < 0.0  then  dc:=0.0; { to prevent strange case }
  ClcVectFromPolinom(dc,ra,va,xc,vc); { uneveras for current vectors }
  ec.epo:=tz;                         { epoch of the elements }
  toeclipt(tz,xc,vc,ec);              { unforequ for kepler ecliptic }
end; { EcliptelMom }

procedure ToTryEphem ( el       : tastel ;   { for set of element }
                       pc       : tcoorec ;  { record for station }
                       tc       : extended ; { current moment }
                       za , zb  : extended ; { moments for interval }
                       ra , va  : tvect3 ) ; { start condition vect }
var
  dc  : Extended ; { difference in day from start moment za }
  tz  : Extended ; { moment with time delay in julian day }
  xc  : tvect3 ;   { current position AU }
  vc  : tvect3 ;   { current velosity AU/day }
  xz  : tvect3 ;   { vector topo station to minor planet equat }
  xp  : tvect3 ;   { vector topo station to minor planet horiz }
  xf  : tvect3 ;   { vector topo station to minor planet fixed }
  az  : Extended ; { azimuth  in degree }
  ah  : Extended ; { altitude in degree }
  ro  : Extended ; { distance in AU with delay }
  aa  : Extended ; { right ascension in degree }
  ad  : Extended ; { declination in degree }
  ec  : tastel ;   { current set of elements }
  bo  : boolean ;  { false if moment out approximation interval }
begin
  if  ( ( tc < za ) or ( (zb+1.0e-8) < tc ) )
    then  Exit ; { moment is out of our interval }
             { may be day 'D' or twilight 'T' on the station }
  if  CharDayOrNight(tc,pc) <> 'N'
    then  Exit ; { only for night result }
  dc:=tc-za; { difference in day from start moment za }
             { minor planet heliocentric pos vel fixed equator }
  ClcVectFromPolinom(dc,ra,va,xc,vc); { uneveras for current vectors }

  if  AstFromEarth(tc,xc) < rangeval  then { unastrng  distance AU }
    begin     { vector for object in topocentric horizontal system }
              { boolean bo is false if moment  out of our interval }
      VectorWithDelay(tc,pc,za,ra,va,tz,xz,xp,bo); { xz xp topocen }
      ClcTopCoorInDegree(xp,az,ah,ro); { UnForCoo azimuth altitude }
      if  ( ah > azimuval ) { result for this height above horizon }
             and  bo        { moment tz  in approximation interval }
        then                { to compare with  altitude  in degree }
          begin
            nprt:=nprt+1; { the next point to print globa unforprt }
            OstatVequat(tc,xz,xf);  { unastrng from terra to fixed }
            SpheriCoorInDegree(xf,aa,ad,ro);   { UnForCoo R.A decl }
            EcliptelMom(tz,za,zb,ra,va,ec);    { ecliptic elements }
            WriteLn(ToGetStrForDate(1,tc),     { function UnStrDat }
                    StrHourA(aa),              { R.A. fixed equato }
                    StrDegrA(2,ad),            { decl fixed equato }
                    ro:9:5,                    { distance in AUnit }
                    az:8:2,ah:7:2,             { azimuth  altitude }
                    togetmau(el,xc,xf):6:1,    { function unastvam }
                    CharDayOrNight(tc,pc):2);  { function UnForDay }
            writeln(fprt,nprt:5,'  ',el.stn);  { p.number and name } 
            writeln(fprt,ToGetStrForDate(3,tc),    { unforprt UnStrDat }
                         togetmau(el,xc,xf):10:1,  { function unastvam }
                         CharDayOrNight(tc,pc):3); { function UnForDay }
            writeln(fprt,StrHourA(aa):18,          { R.A. fixed equato }
                         StrDegrA(2,ad):18,        { decl fixed equato }
                         ro:18:9,                  { distance in AUnit }
                         '  R.A. decl. ro');
            writeln(fprt,StrHourA(az):18,          { azimuth  topohori }
                         StrDegrA(2,ah):18,        { altitude topohori }
                         ro:18:9,                 { distance in AUnit }
                         '  azim altit ro');
            writeln(fprt,ToGetStrForDate(3,tz));   { moment with delay }
            writeln(fprt,ec.ela:18:9,ec.ele:18:9,ec.eli:18:9,'  a e i');
            writeln(fprt,ec.eln:18:9,ec.elp:18:9,ec.elm:18:9,'  O o M');
          end;
    end;
end;

procedure ToTryElems ( el  : tastel ;        { for set of element }
                       tc  : extended ;      { current moment }
                       za , zb  : extended ; { moments for interval }
                       ra , va  : tvect3 ) ; { start condition vect }
var
  dc  : Extended ; { difference in day from start moment za }
  xc  : tvect3 ;   { current position AU }
  vc  : tvect3 ;   { current velosity AU/day }
  ec  : tastel ;   { current set of elements type unastcot }
begin

  if  ( ( tc < za ) or ( (zb+1.0e-8) < tc ) )
    then  Exit ; { moment is out of our interval }

  ec:=el;          { initial eqaul for records }

  EcliptelMom(tc,za,zb,ra,va,ec);  { set of ecliptic elements }

  writeln(ToGetStrForDate(1,ec.epo), { unfordat function }
          ec.ela:14:8,ec.ele:14:8,
          ec.eli:14:7,ec.eln:14:7,ec.elp:14:7); { elements }

end; { totryelems }

{ to screen value for range from the Earth }

procedure ToTryRange ( tc  : extended ;      { current moment }
                       za , zb  : extended ; { moments for interval }
                       ra , va  : tvect3 ) ; { start condition vect }
var
  dc  : Extended ; { difference in day from start moment za }
  xc  : tvect3 ;   { current position AU }
  vc  : tvect3 ;   { current velosity AU/day }
  qq  : Extended ; { value of range in AU }
begin

  if  ( ( tc < za ) or ( (zb+1.0e-8) < tc ) )
    then  Exit ; { moment is out of our interval }
  dc:=tc-za; { difference in day from start moment za }
             { minor planet heliocentric pos vel fixed equator }
  ClcVectFromPolinom(dc,ra,va,xc,vc); { uneveras for current vectors }

  qq:=AstFromEarth(tc,xc); { unastrng  distance AU }

  writeln(ToGetStrForDate(1,tc), { unfordat function }
          qq:12:6);

  writeln(fprt,ToGetStrForDate(1,tc), { unfordat function }
               qq:12:6);

end;

{ to obtain geocentric spheric equator J2000.0 positions }

procedure ToTryRefes ( el       : tastel ;   { for set of element }
                       tc       : extended ; { current moment }
                       za , zb  : extended ; { moments for interval }
                       ra , va  : tvect3 ) ; { start condition vect }
var
  dc  : Extended ; { difference in day from start moment za }
  tz  : Extended ; { moment with time delay in julian day }
  xc  : tvect3 ;   { current position AU }
  vc  : tvect3 ;   { current velosity AU/day }
  xf  : tvect3 ;   { geocentric vector minor planet fixed }
  ro  : Extended ; { distance in AU with delay }
  aa  : Extended ; { right ascension in degree }
  ad  : Extended ; { declination in degree }
  ec  : tastel ;   { current set of elements }
  bo  : boolean ;  { false if moment out approximation interval }
  td  : Extended ; { terrestrial time TT }
  dt  : Extended ; { delta(UT) }
  tb  : Extended ; { barycentric time TDB }
  xe  : tvect3 ;   { pos for the Earth }
  ve  : tvect3 ;   { vel for the Earth }
begin
  if  ( ( tc < za ) or ( (zb+1.0e-8) < tc ) )
    then  Exit ; { moment is out of our interval }
             { may be day 'D' or twilight 'T' on the station }
  dc:=tc-za; { difference in day from start moment za }
             { minor planet heliocentric pos vel fixed equator }
  ClcVectFromPolinom(dc,ra,va,xc,vc); { uneveras for current vectors }
  { descart J2000 position refer to the center of the Earth }
  TransUTCtoTDT(tc,td,dt); { unit unfortim terrestrial time TT }
  TransTDTtoTDB(td,tb);    { barycentric time TDB Teph for ephemeris }
  earths(tb,xe,ve);        { unephjpl the Earth from the Sun vector }
  xf[1]:=xc[1]-xe[1];      { to the center of the Earth x position }
  xf[2]:=xc[2]-xe[2];      { to the center of the Earth y position }
  xf[3]:=xc[3]-xe[3];      { to the center of the Earth z position }
  { with light delay in day constants from UnAstCoT }
  tz:=tc-VectorModul(xf)*AstrUnit/(86400.0*VelOfLight);
  if  tz < za    { moment may be out of our interval }
    then  Exit ; { of approximation boolean is false }
  dc:=tz-za; { difference in day from start moment za }
  ClcVectFromPolinom(dc,ra,va,xc,vc); { uneveras }
  xf[1]:=xc[1]-xe[1];      { to the center of the Earth x position }
  xf[2]:=xc[2]-xe[2];      { to the center of the Earth y position }
  xf[3]:=xc[3]-xe[3];      { to the center of the Earth z position }
  { with light delay in day constants from UnAstCoT }
  tz:=tc-VectorModul(xf)*AstrUnit/(86400.0*VelOfLight);
  if  tz < za    { moment may be out of our interval }
    then  Exit ; { of approximation boolean is false }
  dc:=tz-za; { difference in day from start moment za }
  ClcVectFromPolinom(dc,ra,va,xc,vc); { uneveras }
  xf[1]:=xc[1]-xe[1];      { to the center of the Earth x position }
  xf[2]:=xc[2]-xe[2];      { to the center of the Earth y position }
  xf[3]:=xc[3]-xe[3];      { to the center of the Earth z position }
  SpheriCoorInDegree(xf,aa,ad,ro);   { UnForCoo R.A decl }
  WriteLn(ToGetStrForDate(1,tc),     { function UnStrDat }
          StrHourA(aa),              { R.A. fixed equato }
          StrDegrA(2,ad),            { decl fixed equato }
          ro:13:7,                    { distance in AUnit }
          togetmau(el,xc,xf):9:2);   { function unastvam }
end;

{ to obtain some values
  during small interval of integration [ za , zb ] }

procedure SingleAstResult ( ia  : Byte ;          { index what to do }
                            el  : tastel ;        { for set of element }
                            pc  : tcoorec ;       { record for station }
                            tc  : extended ;      { curent moment }
                            za , zb  : extended ; { moments for interval }
                            ra , va  : tvect3 ) ; { start condition vect }
begin

case  ia  of                            { index what to do }
 0 :  ToTryEphem(el,pc,tc,za,zb,ra,va); { ephemeris for station }
 1 :  ToTryElems(el,tc,za,zb,ra,va);    { ecliptic elements }
 2 :  ToTryRange(tc,za,zb,ra,va);       { range from the Earth }
 3 :  ToTryRefes(el,tc,za,zb,ra,va);    { geocentric astrometric J2000 }
end ;                                   { case index what to do }

end;

end.
