
unit UnForEqu ; { from ecliptic elements to fixed equator }

interface

uses
  unastcot ; { for type tvect3 tastel }

{ to obtain heliocentric ecliptical
   descart position in AU velosity in AU/day }

procedure tocartus ( el  : tastel ;  var  pc , vc  : tvect3 ) ;

{ from ecliptic to equator
   input 
   el record elements
      heliocentric ecliptic
   output
      heliocentric equatorial
   minor planet
   position in AU , velosity in AU/day }

procedure toequars ( el  : tastel ;  var  xp , vp  : tvect3 ) ;

{ from equator to ecliptic
   input 
      heliocentric equatorial
   minor planet
   position in AU , velosity in AU/day
   output
   el record elements
      heliocentric ecliptic }

procedure toeclipt ( tc       : extended ; { moment in julian day }
                     pc , vc  : tvect3 ;   { equator position velosity }
                     var  el  : tastel ) ; { record ecliptic elements }

implementation

uses
  unfornut , { function epsmean for \epsilon obliquity }
  unforkep ; { procedure fromkeplertus }

{ to obtain heliocentric ecliptical
   descart position in AU velosity in AU/day }

procedure tocartus ( el  : tastel ;  var  pc , vc  : tvect3 ) ;
var
  pa,ua  : tvect3 ; (* position and angle variables *)
begin
  with  el  do    (* records with elements for current minor planet *)
    begin         { heliocentric ecliptical system }
      pa[1]:=ela; (* a in AU *)
      pa[2]:=ele; { eccentricity }
      pa[3]:=eli; (* i in degree *)
      ua[1]:=eln; { the ascending node ecliptical }
      ua[2]:=elp; { the argument of perihelium ecliptical }
      ua[3]:=elm; (* M in degree ecliptical *)
    end;
  fromkeplertus(pa,ua,pc,vc); (* from unkp position and velosity *)
end;

{ from ecliptic to equator
   input 
   el record elements
      heliocentric ecliptic
   output
      heliocentric equatorial
   minor planet
   position in AU , velosity in AU/day }

procedure toequars ( el  : tastel ;  var  xp , vp  : tvect3 ) ;
var
  pc,vc  : tvect3 ;
  ep,se,ce  : extended ;
begin
  tocartus(el,pc,vc);  (* cartesian but ecliptical AU AU/day *)
  ep:=epsmean(el.epo); (* unfornut \epsilon obliquity in radian *)
  se:=sin(ep);         { \sin \epsilon obliquity }
  ce:=cos(ep);         { \cos \epsilon obliquity }
  xp[1]:=+pc[1]; (* rotation around x-axis on angle (-epsilon) *)
  xp[2]:=+ce*pc[2]-se*pc[3]; (* equatorial position in AU *)
  xp[3]:=+se*pc[2]+ce*pc[3]; (* reffered to the Sun *)
  vp[1]:=+vc[1]; (* rotation around x-axis on angle -epsilon *)
  vp[2]:=+ce*vc[2]-se*vc[3]; { equatorial velosity in AU/day }
  vp[3]:=+se*vc[2]+ce*vc[3]; { reffered to the Sun heliocentric }
end;

{ from equator to ecliptic J2000
   input 
      heliocentric equatorial
   minor planet
   position in AU , velosity in AU/day
   output
   el record elements
      heliocentric ecliptic }

procedure toeclipt ( tc       : extended ; { moment in julian day }
                     pc , vc  : tvect3 ;   { equator position velosity }
                     var  el  : tastel ) ; { record ecliptic elements }
var
  xp,vp  : tvect3 ;      { ecliptic position velosity AU AU/day }
  ep,eu  : tvect3 ;      { ecliptic elements a e i O o M }
  op,se,ce  : extended ; { obliquity sinus cosinus }
begin
  op:=epsmean(jd2000); { unfornut \epsilon obliquity in radian }
  se:=sin(op);     { \sin \epsilon obliquity }
  ce:=cos(op);     { \cos \epsilon obliquity }
  xp[1]:=+pc[1];   { rotation around x-axis on angle (+epsilon) }
  xp[2]:=+ce*pc[2]+se*pc[3]; { ecliptic position in AU }
  xp[3]:=-se*pc[2]+ce*pc[3]; { reffered to the Sun }
  vp[1]:=+vc[1];   { rotation around x-axis on angle +epsilon }
  vp[2]:=+ce*vc[2]+se*vc[3]; { equatorial velosity in AU/day }
  vp[3]:=-se*vc[2]+ce*vc[3]; { reffered to the Sun heliocentric }
  ToGetKeplerElements(xp,vp,ep,eu); { UnForKep elements }
  with  el  do
    begin                           { type record UnAstCoT }
      epo:=tc;    { epoch of the elements }
      ela:=ep[1]; { a semimajor axis AU }
      ele:=ep[2]; { e eccentricity }
      eli:=ep[3]; { i inclination in degree }
      eln:=eu[1]; { O the ascending node degree }
      elp:=eu[2]; { o the argument of perihelium degree }
      elm:=eu[3]; { M the mean anomaly in degree }
    end;
end;

end.
