
{ to read text file with test data }

unit unreagau ;

interface

uses
  unastcot ; { type tvect3 }

procedure unoveclipt ( n  : integer ;       { number in file }
                       s  : string ;        { name of file }
                       x  : extended ;      { value for error arcsec }
                       var  t  : extended ; { moment for observation }
                       var  r  : extended ; { distance AU }
                       var  e  : tvect3 ;   { unit p ecliptic helios }
                       var  z  : tastel ;   { record for elements }
                       var  y  : terand ) ; { for random error }

implementation

uses
  unfordat , { function fromdatetojd }
  unforfun , { function withgaussvalue }
  unfornut ; { function epsmean }

{ input
  n number of point in testgaus.txt file
  output
  t moment for observation
  e unit vector ecliptic system heliocentric }

procedure unoveclipt ( n  : integer ;       { number in file }
                       s  : string ;        { name of file }
                       x  : extended ;      { value for error arcsec }
                       var  t  : extended ; { moment for observation }
                       var  r  : extended ; { distance AU }
                       var  e  : tvect3 ;   { unit p ecliptic helios }
                       var  z  : tastel ;   { record for elements }
                       var  y  : terand ) ; { for random error }
var
  nr  : integer ;  { count for read }
  fr  : text ;     { file with observations }
  ny  : integer ;  { year }
  nm  : integer ;  { month }
  nd  : integer ;  { day }
  kh  : integer ;  { hour }
  km  : integer ;  { minute }
  ts  : extended ; { second }
  ih  : integer ;  { R.A. hour }
  im  : integer ;  { R.A. minute }
  az  : extended ; { R.A. second }
  md  : integer ;  { decl degree }
  mm  : integer ;  { decl minute }
  ds  : extended ; { decl second }
  ro  : extended ; { distance AU }
  ra  : extended ; { right ascension in radian }
  rd  : extended ; { declination in radian }
  uq  : tvect3 ;   { unit equator vector unastcot }
  op  : extended ; { \epsilon_0 }
  se  : extended ; { sin(\epsilon) }
  ce  : extended ; { cos(\epsilon) }
  st  : string[24] ; { temporary to read text file }
  ld  : integer ;  { altitude degree }
  lm  : integer ;  { altitude minute }
begin
  assign(fr,s); { file with simple test }
  reset(fr);    { to open to read }
  nr:=0;
repeat
  readln(fr);                   { nullo string }
  readln(fr,ny,nm,nd,kh,km,ts); { year month day hour min sec }
  readln(fr,ih,im,az,md,mm,ds,  { right ascension declination }
            ro);                { distance AU }
  readln(fr,st,ld,lm);          { azimuth altitude distance }
  nr:=nr+1;
  if  nr = n  then              { it is our number of observation }
    begin

      y.alti:=ld+lm/60.0; { altitude in degree }

      t:=fromdatetojd(nd,nm,ny,kh,km,ts); { julian day unfordat }
      ra:=15.0*grarad*(abs(ih)+(im+az/60.0)/60.0); { r.a. radian }
      if  ih < 0  then  ra:=-ra; { may be negative value r.a. }

      y.eras:=withgaussvalue(0.0,x);   { unforfun r.a. arcsec }
      ra:=ra+secrad*y.eras; { unastcot random error to radian }

      rd:=grarad*(abs(md)+(mm+ds/60.0)/60.0);   { decl radian }
      if  md < 0  then  rd:=-rd; { may be negative value decl }

      y.erds:=withgaussvalue(0.0,x);   { unforfun decl arcsec }
      rd:=rd+secrad*y.erds; { unastcot random error to radian }

      uq[1]:=cos(rd)*cos(ra);    { unit vector x equator }
      uq[2]:=cos(rd)*sin(ra);    { unit vector y equator }
      uq[3]:=sin(rd);            { unit vector z equator }
      op:=epsmean(jd2000); { unfornut \epsilon obliquity in radian }
      se:=sin(op);  { \sin \epsilon obliquity }
      ce:=cos(op);  { \cos \epsilon obliquity }
      e[1]:=+uq[1]; { rotation around x-axis on angle (+epsilon) }
      e[2]:=+ce*uq[2]+se*uq[3]; { ecliptic position }
      e[3]:=-se*uq[2]+ce*uq[3]; { unit vector }
      r:=ro; { output distance AU }
      readln(fr,ny,nm,nd,kh,km,ts); { year month day hour min sec }
      z.epo:=fromdatetojd(nd,nm,ny,kh,km,ts); { date moment with delay }
      readln(fr,z.ela,z.ele,z.eli);           { kepler elements a e i }
      readln(fr,z.eln,z.elp,z.elm);           { kepler elements O o M }
    end       else              { to repeat to read file }
    begin
      readln(fr); { date moment with delay }
      readln(fr); { kepler elements a e i }
      readln(fr); { kepler elements O o M }
    end;
until  EOF(fr) or ( nr = n ) ; { may be end of file }
  close(fr);                   { that is all }
end;

end.
