
(*
 station position type from UnAstCoT

 type
   TCooRec   = record
                 nums  : Integer ;    { number in list }
                 name  : string[12] ; { name of station }
                 desc  : tvect3 ;     { position in kilometer }
                 vauc  : tvect3 ;     { position in Astronomical unit }
                 geod  : typedim3 ;   { lat long degree h kilometer }
                 topm  : tmatr33 ;    { matrix to change }
               end;
*)

unit UnStaCoo ;

interface

uses
  UnAstCoT ; { const RadGra GraRad type tcoorec }

procedure GeodFromDesc ( z  : tvect3 ;
                         var  u  : typedim3 ) ;

procedure DescFromGeod ( u  : typedim3 ;
                         var  z  : tvect3 ) ;

procedure TopoHoriMatr ( var  p  : tcoorec ) ; { type UnAstCoT }

{ to select station position
     by using text file pointcoo.txt
  input  n number station in file pointcoo.txt
  output p record to indentify station with coordinates }

procedure PointCoor ( n  : Integer ;  var  p  : tcoorec ) ;

implementation

uses
  UnForFun ; { functions DATan2 ATanDegree }

{ to obtain geodetic coordinates ( h in kilometer )
  input
         z[1]  descart position x in km
         z[2]  descart position y in km
         z[3]  descart position z in km
  output
         u[1]  latitude  in degree
         u[2]  longitude in degree
         u[3]  height    in kilometer
     of the station from the cartesian ones in kilometer }

procedure GeodFromDesc ( z  : tvect3 ;          { descart }
                         var  u  : typedim3 ) ; { geodeti }
var
  irf                : Byte ;
  a,e,r,
  pp,hg,gg,sv,cv,
  sf,cf,sff,cff,cc   : Extended ;
begin
  r:=6378.14411;     { in kilometer }
  a:=1.0/298.257;    { oblateness }
  e:=2*a-a*a;        { eccentricity }
  pp:=Sqrt(Sqr(z[1])+Sqr(z[2])); { \sqrt(x^2+y^2) }
  cv:=z[1]/pp;          { \cos(\labmda) }
  sv:=z[2]/pp;          { \sin(\lambda) }
  hg:=0.0;
  For irf:=1 To 4 Do { iterations to find }
    begin            { latitude and height }
      sff:=z[3]*(1+hg);
      cff:=pp*(1-e+hg);
      cc:=Sqrt(sff*sff+cff*cff);
      sf:=sff/cc;                { \sin(\phi) }
      cf:=cff/cc;                { \cos(\phi) }
      gg:=r/Sqrt(1-e*sf*sf);
      hg:=pp/(gg*cf)-1.0;
    End;
  u[1]:=RadGra*DATan2(sf,cf); { latitude  in degree UnForFun }
  u[2]:=ATanDegree(sv,cv);    { longitude in degree UnForFun }
  u[3]:=hg*gg;                { height    in kilometer }
end;

{ to obtain
     cartesian coordinates in kilometer
  input
         u[1]  latitude  in degree
         u[2]  longitude in degree
         u[3]  height    in kilometer
  output
         z[1]  descart position x in km
         z[2]  descart position y in km
         z[3]  descart position z in km
     of the station from the geodetic ones }

procedure DescFromGeod ( u  : typedim3 ;      { geodeti }
                         var  z  : tvect3 ) ; { descart }
var
  a,e,r,q,
  ff,vv,sf,cf,gg  : Extended ;
begin
  r:=6378.14411;   { in kilometer }
  a:=1.0/298.257;  { oblateness }
  e:=2*a-a*a;      { eccentricity by oblateness }
  q:=1-e;          { temporary }
  ff:=GraRad*u[1]; { latitude radian }
  vv:=GraRad*u[2]; { longitude radian }
  sf:=Sin(ff);           { sinus latitude }
  cf:=Cos(ff);           { cosinus latitude }
  gg:=r/Sqrt(1-e*sf*sf);    { temporary }
  z[1]:=(gg+u[3])*cf*Cos(vv);  { descart position x kilometer }
  z[2]:=(gg+u[3])*cf*Sin(vv);  { descart position y kilometer }
  z[3]:=(gg*q+u[3])*sf;        { descart position z kilometer }
end;

{ to obtain matrix
  to change to topocentric horizont
  simple rotation around Z on angle \lambda
  simple rotation ariund Y on angle 90-\phi }

procedure TopoHoriMatr ( var  p  : tcoorec ) ; { type UnAstCoT }
var
  a,b  : tmatr33 ; { type UnAstCoT simple matrix }
begin                                      { procedures UnForFun }
  ForRotMatr(3,grarad*p.geod[2],a);        { R_3(\lambda) around Z }
  ForRotMatr(2,grarad*(90.0-p.geod[1]),b); { R_2(90-\phi) around Y }
  ToMultMatr(b,a,p.topm); { to horizontal system OX to south }
end;

{ to select station position
     by using text file pointcoo.txt
  input  n number station in file pointcoo.txt
  output p record to indentify station with coordinates }

procedure PointCoor ( n  : Integer ;  var  p  : tcoorec ) ; { UnAstCoT }
var
  f  : Text ;    { file with sites positions  pointcoo.txt }
  i  : Byte ;    { smple count }
  b  : Boolean ; { true for our station }
begin
  Assign(f,'pointcoo.txt'); { text file with station coordinates }
  ReSet(f);                 { to open to read }
  ReadLn(f);                { the first  text string }
  ReadLn(f);                { the second text string }
  b:=False;                 { initial to choose stat }
repeat                      { to fill part of record }
  ReadLn(f,p.nums,                          { station number }
           p.desc[1],p.desc[2],p.desc[3],   { station position }
           p.name);                         { station name }
  if  p.nums = n then                       { to compare numbers }
    begin                                   { this is our point }
      p.desc[1]:=1.0e-3*p.desc[1];          { x in km }
      p.desc[2]:=1.0e-3*p.desc[2];          { y in km }
      p.desc[3]:=1.0e-3*p.desc[3];          { z in km }
      b:=True; { our choice }
    end;
until  b or EOF(f) ; { may be end of file }
  Close(f); { that is all }
  GeodFromDesc(p.desc,p.geod);  { \phi \lambda h in km }
  TopoHoriMatr(p); { to change to topocentric horizont }
  for i:=1 to 3 do { for all three station coordinates }
    p.vauc[i]:=p.desc[i]/astrunit; { from kilometer AU }
end;                                     

end.
