unit untreran ; { for ro_1 ro_2 ro_3 in Gauss method }
interface
uses
  unastcot ; { type tvect3 }
{ a try to estimate three value for topocentric range }
procedure trerange ( t1 , t2 , t3  : extended ;        { moments }
                     e1 , e2 , e3  : tvect3 ;          { unit vectors }
                     s1 , s2 , s3  : tvect3 ;          { the Sun }
                     h1 , h2 , h3  : extended ;        { for \eta }
                     var  q1 , q2 , q3  : extended ) ; { result ro }
{ to obtain three vector by using three heliocentric range }
procedure trevecto ( e1 , e2 , e3  : tvect3 ;        { unit vectors }
                     s1 , s2 , s3  : tvect3 ;        { the Sun vect }
                     q1 , q2 , q3  : extended ;      { ro distances }
                     var  r1 , r2 , r3  : tvect3 ) ; { eclip helios }
{ to obtain value for ratio \eta }
function togeteta ( ta , tb  : extended ; { two moments tb > ta }
                    qa , qb  : extended ; { two distance }
                    ra , rb  : tvect3 ) : extended ; { helioecl }
implementation
uses
  unforfun ; { procedure VectorMult }
{ to obtain three heliocentric range for three point }
procedure trerange ( t1 , t2 , t3  : extended ;        { moments }
                     e1 , e2 , e3  : tvect3 ;          { unit vectors }
                     s1 , s2 , s3  : tvect3 ;          { the Sun }
                     h1 , h2 , h3  : extended ;        { for \eta }
                     var  q1 , q2 , q3  : extended ) ; { result ro }
var
  d1  : tvect3 ;   { vector multiplication [ e_2 x e_3 ] }
  d2  : tvect3 ;   { vector multiplication [ e_3 x e_1 ] }
  d3  : tvect3 ;   { vector multiplication [ e_1 x e_2 ] }
  dd  : extended ; { determinant }
  u1  : extended ; { ratio n_1=\tau_1/\tau_2 }
  u3  : extended ; { ratio n_3=\tau_3/\tau_2 }
var
  d11,d12,d13  : extended ; { simple matrix elements }
  d21,d22,d23  : extended ; { simple matrix elements }
  d31,d32,d33  : extended ; { simple matrix elements }
  ta1,ta2,ta3  : extended ; { time differencies }
  za1,za2,za3  : extended ; { moment with delay }
  et1,et2,et3  : extended ; { for parameter eta }
begin
  za1:=t1-0.005776*q1;       { moment with delay }
  za2:=t2-0.005776*q2;       { moment with delay }
  za3:=t3-0.005776*q3;       { moment with delay }
  ta1:=gaussconst*(za3-za2); { unastcot time difference }
  ta2:=gaussconst*(za3-za1); { unastcot time difference }
  ta3:=gaussconst*(za2-za1); { unastcot time difference }
  vectormult(e2,e3,d1);   { unforfun }
  vectormult(e3,e1,d2);   { unforfun }
  vectormult(e1,e2,d3);   { unforfun }
  dd:=scalarmult(e1,d1);  { unforfun }
  d11:=scalarmult(d1,s1); { for the Sun }
  d12:=scalarmult(d1,s2); { for the Sun }
  d13:=scalarmult(d1,s3); { for the Sun }
  d21:=scalarmult(d2,s1); { for the Sun }
  d22:=scalarmult(d2,s2); { for the Sun }
  d23:=scalarmult(d2,s3); { for the Sun }
  d31:=scalarmult(d3,s1); { for the Sun }
  d32:=scalarmult(d3,s2); { for the Sun }
  d33:=scalarmult(d3,s3); { for the Sun }
  et1:=h1; { initial nullo estimation }
  et2:=h2; { initial nullo estimation }
  et3:=h3; { initial nullo estimation }
  u1:=(et2/et1)*(ta1/ta2); { ratio n_1=(\eta_2/\eta_1)*(\tau_1/\tau_2) }
  u3:=(et2/et1)*(ta3/ta2); { ratio n_3=(\eta_2/\eta_3)*(\tau_3/\tau_2) }
  q1:=(u1*d11-d12+u3*d13)/(u1*dd); { estimation \ro_1 AU }
  q2:=-(u1*d21-d22+u3*d23)/dd;     { estimation \ro_2 AU }
  q3:=(u1*d31-d32+u3*d33)/(u3*dd); { estimation \ro_3 AU }
end;
{ to obtain three vector by using three heliocentric range }
procedure trevecto ( e1 , e2 , e3  : tvect3 ;        { unit vectors }
                     s1 , s2 , s3  : tvect3 ;        { the Sun vect }
                     q1 , q2 , q3  : extended ;      { ro distances }
                     var  r1 , r2 , r3  : tvect3 ) ; { eclip helios }
var
  i  : byte ; { count for three position }
begin
  for i:=1 to 3 do
    begin
      r1[i]:=q1*e1[i]-s1[i]; { the first  vector heliocentric esliptic }
      r2[i]:=q2*e2[i]-s2[i]; { the second vector heliocentric esliptic }
      r3[i]:=q3*e3[i]-s3[i]; { the third  vector heliocentric esliptic }
    end;
end;
{ function for method of Newton }
function functionear ( et , vm , vl  : extended ) : extended ;
var
  n  : integer ; { simple count }
  a,b,s,c,u,w  : extended ; { temporary }
begin
  a:=vm/Sqr(et); { m/\eta^2 }
  b:=a-vl;       { m/\eta^2-l }
  if  abs(b) < 0.1
    then         { approximate by series }
      begin
        w:=4.0/3.0; { the first term in series }
        c:=w;       { the first term }
        u:=4.0;     { for factor }
        n:=0;
        while  ( abs(c) > 1.0e-14 )
                 and ( n < 10 )  do { to add term by term }
          begin                     { to our series }
            n:=n+1;
            u:=u+2.0;               { for the factor }
            c:=c*(u/(u-1.0))*b;      { the next term ins series }
            w:=w+c;                 { to add the next term }
          end;
      end
    else
      if  b > 0.0  { positive }
        then
          begin
            s:=sqrt(b);               { sin }
            c:=sqrt(abs(1.0-sqr(s))); { cosinus }
            c:=2.0*datan2(s,c);       { unforfun }
            u:=sin(c);
            w:=(2.0*c-sin(2.0*c))/(u*sqr(u));
          end
        else       { negative }
          begin
            c:=2.0*ln(sqrt(-b)+sqrt(1-b)); { Ln instead of Arsh }
            u:=0.5*(exp(c)-exp(-c));       { sh(c) hyperbolic }
            w:=(0.5*(exp(2.0*c)-exp(-2.0*c))-2.0*c)/(u*sqr(u));
          end;
  functionear:=1.0-et+a*w; { result function for \eta }
end;
{ to obtain value for ratio \eta }
function togeteta ( ta , tb  : extended ; { two moments tb > ta }
                    qa , qb  : extended ; { two distance }
                    ra , rb  : tvect3 ) : extended ; { helioecl }
var
  nc  : integer ;  { count }
  za  : extended ; { moment ta with delay }
  zb  : extended ; { moment tb with delay }
  tu  : extended ; { \tau time interval }
  da  : extended ; { modul vector ra }
  db  : extended ; { modul vector rb }
  ab  : extended ; { scalar multiplication (a*b) }
  qq  : extended ; { temporary value }
  vl  : extended ; { value l }
  vm  : extended ; { value m }
  v1  : extended ; { the first  value \eta for iteration }
  v2  : extended ; { the second value \eta for iteration }
  vz  : extended ; { minimum value for \eta }
  dv  : extended ; { for \eta correction }
  f1  : extended ; { function for method of Newton }
  f2  : extended ; { function for method of Newton }
begin
  za:=ta-0.005776*qa ;    { moment ta with delay }
  zb:=tb-0.005776*qb ;    { moment ta with delay }
  tu:=gaussconst*(zb-za); { unastcot \tau time interval with delay }
  da:=vectormodul(ra);    { unforfun }
  db:=vectormodul(rb);    { unforfun }
  ab:=scalarmult(ra,rb);  { unforfun }
  qq:=2.0*(da*db+ab);     { 2*(r_a*r_b+(\vec r_a*\vec r_b)) }
  vl:=0.5*(da+db)/sqrt(qq)-0.5; { value l }
  vm:=sqr(tu)/(qq*sqrt(qq));    { value m }
  vz:=sqrt(vm/(1.0+vl));        { minimum value for \eta }
  { Hansen approximative for \eta with l m values }
  v2:=12.0/22.0+(10.0/22.0)*sqrt(1+(44.0/9.0)*(vm/(vl+5.0/6.0)));
  v1:=v2+0.1; { approximate the first and second values \eta }
  f1:=functionear(v1,vm,vl); { function for method of Newton }
  f2:=functionear(v2,vm,vl); { function for method of Newton }
  nc:=0; { count for iteration }
  while  ( abs(f2-f1) > 1.0e-14 ) and ( nc < 10 )  do
    begin
      dv:=-f2*(v2-v1)/(f2-f1); { correction for f(\eta) = 0 }
      v1:=v2;
      f1:=f2;
      while  (v2+dv) <= vz  do  dv:=0.5*dv; { to small corection }
      v2:=v2+dv;                 { new value \eta }
      f2:=functionear(v2,vm,vl); { new value function }
      nc:=nc+1;                  { the next step }
    end;
  togeteta:=v2; { result }
end;
end.
