unit untrerel ; { elemnts from two position vectors }
interface
uses
  unastcot ; { type tvect3 tastel }
{ two position vectors are known
  in ecliptical heliocentric system
  to calculate the set of elements }
procedure ElemFromTwoPos ( t1 , t3  : extended ; { two moments }
                           y1       : extended ; { range in AU }
                           h2       : extended ; { value  \eta }
                           r1 , r3  : tvect3 ;   { two vectors }
                           var  el  : tastel ;   { result elem }
                           var  bo  : boolean ) ; { may be err }
implementation
uses
  unforfun ; { functions scalarmult vectormodul }
{ two position vectors are known
  in ecliptical heliocentric system
  to calculate the set of elements }
procedure ElemFromTwoPos ( t1 , t3  : extended ; { two moments }
                           y1       : extended ; { range in AU }
                           h2       : extended ; { value  \eta }
                           r1 , r3  : tvect3 ;   { two vectors }
                           var  el  : tastel ;   { result elem }
                           var  bo  : boolean ) ; { may be err }
var
  i   : byte ;     { simple count }
  ea  : tvect3 ;   { unit vector for \vec r_1 }
  eo  : tvect3 ;   { unit vector for \vec r_0 }
  en  : tvect3 ;   { unit vector for \Omega }
  r0  : tvect3 ;   { vector r_3-(r_3*e_a)*e_a }
  rp  : tvect3 ;   { unit vector perpendicular to place }
  si  : extended ; { sinus   inclination }
  ci  : extended ; { cosinus inclination }
  sn  : extended ; { sinus   ascending node }
  cn  : extended ; { cosinus ascending node }
  su  : extended ; { sinus   argument of latitude }
  cu  : extended ; { cosinus argument of latitude }
  uu  : extended ; { value u_1 = v_1 + \omega }
  rr  : extended ; { the area of triangle }
  pp  : extended ; { parametr p=a*(1-e^2) }
  ce  : extended ; { e*\cos v_1 and \cos E }
  se  : extended ; { e*\sin v_1 and \sin E }
  ve  : extended ; { value E in radian }
  cv  : extended ; { \cos v_1 }
  sv  : extended ; { \sin v_1 }
  vv  : extended ; { value v_1 true anomaly }
  qq  : extended ; { temporary value }
  q0  : extended ; { modul  \vec r_0 }
  q1  : extended ; { modul  \vec r_1 }
  q3  : extended ; { modul  \vec r_3 }
  qz  : extended ; { temporary value }
begin
  bo:=false; { default for error in calculations }
  el.epo:=t1-0.005776*y1; { epoch of the elemnts with delay }
  unitvector(r1,ea);  { \vec r_1/!\vec r_1! unforfun procedure }
  qq:=scalarmult(r3,ea);  { value (\vec r_3*\vec e_a) unforfun }
  for i:=1 to 3 do        { for each coordinate for new vector }
    r0[i]:=r3[i]-qq*ea[i]; { new vector in place perpendicular }

  if  vectormodul(r0) < 1.0e-6  then  exit ; { non-real vector }

  unitvector(r0,eo);       { new vector to unitvector in place }
  vectormult(ea,eo,rp);    { unit vector perpendicula to place }

  ci:=rp[3];       { cosinus inclination }
  qz:=1.0-sqr(ci); { value for sin^2 }
  if  qz < 0.0     { may be negative }
    then  exit ;   { error }
  si:=sqrt(qz);    { sinus plus always }

  sn:=+rp[1]/si;         { sinus   ascending node }
  cn:=-rp[2]/si;         { cosinus ascending node }

  el.eli:=ATanDegree(si,ci); { UnForFun inclination in degr }

  el.eln:=ATanDegree(sn,cn); { UnForFun ascending n in degr }

  en[1]:=cn;  { unit vector with ascending node \cos \Omega }
  en[2]:=sn;  { unit vector with ascending node \sin \Omega }
  en[3]:=0.0; { unit vector with ascending node nullo }
  cu:=+scalarmult(ea,en); { unforfun \cos (u_1) }
  su:=-scalarmult(eo,en); { unforfun \sin (u_1) }
  uu:=ATanDegree(su,cu);  { value u_1 in degree }
  q0:=vectormodul(r0);    { for \vec r_0 }
  q1:=vectormodul(r1);    { for \vec r_1 }
  q3:=vectormodul(r3);    { for \vec r_3 }
  rr:=0.5*q1*q0;          { the area \Delta param pp }
  pp:=sqr((2.0*rr*h2)/(gaussconst*(t3-t1))); { unastcot time difference }
  ce:=pp/q1-1.0; { e\cos v_1 eccentricity and true anomaly }
  se:=(ce*(qq/q3)-(pp/q3-1.0))/(q0/q3);   { for e*\sin v_1 }

  el.ele:=sqrt(sqr(se)+sqr(ce)); { eccentricity value }

  if  el.ele < 1.0e-6  then  exit ; { non-real eccentricity value }

  qz:=1.0-sqr(el.ele); { 1-e^2 }

  if  qz <= 0.0  then  exit ; { error }

  el.ela:=pp/qz;  { semimajor axis a=p/(1-e^2) }

  if  el.ela > 3.5  then  exit ;  { non-real semimajor axis value }

  cv:=ce/el.ele;         { \cos \v_1 true anomaly }
  sv:=se/el.ele;         { \sin \v_1 true anomaly }
  vv:=ATanDegree(sv,cv); { value v_1 in degree }

  el.elp:=uu-vv;         { argument of perihelium }
  if  el.elp < 0.0              { may be negative value }
    then  el.elp:=360.0+el.elp; { to positive in degree }

  se:=(sqrt(qz)*sv)/(1.0+el.ele*cv); { for \sin E }
  ce:=(cv+el.ele)/(1.0+el.ele*cv);   { for \cos E }
  ve:=datan2(se,ce); { value of E in radian }

  el.elm:=radgra*(ve-el.ele*se); { value M mean anomaly degree }
  bo:=true; { no error in calculations }
end;
end.
